#include "jserver.h"

resource resources = NULL;

resource resource_create(int type, char *user, char *res, pth_msgport_t handler)
{
    resource new;

    debug("RESOURCE:ADD %d %s/%s %d\n",type,user,res,handler);
    new = malloc(sizeof(_resource));
    new->type = type;
    new->user = j_strdup(user);
    new->res = j_strdup(res);
    new->handler = handler;
    new->next = resources;
    resources = new;

    return new;
}

resource resource_lookup(char *user, char *res)
{
    resource next;

    if(res == NULL)
        return NULL;

    next = resources;
    while(next != NULL)
    {
/*        printf("RESOURCE:CHECK %d %s/%s %d\n",next->type,next->user,next->res,next->handler);*/
        if((j_strcmp(user, next->user) == 0 || user == NULL) && j_strcmp(res, next->res) == 0)
            return next;
        next = next->next;
    }
    return NULL;
}

resource resource_list(char *user)
{
    static resource next = NULL;

    if(user == NULL)
        return NULL;

    /* since next is static, reset if it was last at the end or is unset, else move past our last result */
    if(next == NULL)
        next = resources;
    else
        next = next->next;
    
    while(next != NULL)
    {
        if(j_strcmp(user, next->user) == 0)
            break;
        next = next->next;
    }
    return next;
}

void resource_free(resource r)
{
    resource next;

    if(r == NULL)
        return;
    
    if(resources == r)
    {
        resources = r->next;
    }else{
        next = resources;
        while(next != NULL)
        {
            if(next->next == r)
            {
                next->next = next->next->next;
                break;
            }
            next = next->next;
        }
    }

	debug("Freeing user: %s, res: %s", r->user, r->res);
    free(r->user);
    free(r->res);
    free(r);
}

/* NOTE: any jpacket sent to deliver *MUST* match jpacket_new(p->x), 
 * jpacket is simply a convenience wrapper
 */
void deliver(jpacket p)
{
    resource lookup;
    static pth_msgport_t unknown_mp = NULL;
    static pth_msgport_t external_mp = NULL;
    static pth_msgport_t server_mp = NULL;
    int handled = 0;
    udata u;

    debug("deliver(to[%s],from[%s],type[%d])",jid_full(p->to),jid_full(p->from),p->type);

    if(unknown_mp == NULL)
        unknown_mp = pth_msgport_find("unknown");
    if(external_mp == NULL)
        external_mp = pth_msgport_find("external");
    if(server_mp == NULL)
        server_mp = pth_msgport_find("server");

    /* important main function, takes any packet and attempts to deliver it to the correct session/thread */
    /* calls resource_lookup, creates/sends message, etc */

    if(p->to == NULL)
    {
        warn("delivery","no recipient, bouncing");
        /* XXX should add some error stuff here and bounce
        p->to = p->from; */
        return;
    }

    /* first, is it destined externally? */
    if(config_names(CMD_CHECK,p->to->server) == NULL)
    {
        psend(external_mp,p);
        return;
    }

    /* is it for us? */
    if(p->to->user == NULL && p->to->resource == NULL && p->to->server != NULL)
    {
        psend(server_mp,p);
        return;
    }

    /* is this for a known resource? */
    lookup = resource_lookup(p->to->user, p->to->resource);
    if(lookup != NULL)
    {
        psend(lookup->handler,p);
        return;
    }

    /* data sent to a user and no specific resource, deliver intelligently */
    if(p->to->user != NULL && p->to->resource == NULL)
    {
        if(p->type == JPACKET_PRESENCE)
        {
            /* hack: general incoming presence to a user on this server, track global availability flags */
            if(j_strcmp(xmlnode_get_attrib(p->x,"type"),"available") == 0)
                presence_check(CMD_SET,xmlnode_get_attrib(p->x,"from"));
            if(j_strcmp(xmlnode_get_attrib(p->x,"type"),"unavailable") == 0)
                presence_check(CMD_DEL,xmlnode_get_attrib(p->x,"from"));

            /* broadcast it to all the resources for this user */
            while((lookup = resource_list(p->to->user)) != NULL)
            {
                handled = 1;
                psend(lookup->handler,jpacket_new(xmlnode_dup(p->x)));
            }
            if(handled)
            {
                xmlnode_free(p->x); /* only copies have been delivered, delete the orignal */
                return;
            }
        }
        if(p->type == JPACKET_MESSAGE)
        {
            /* find the primary session if any */
            u = user_data(p->to->user);
            if(u->primary != NULL)
            {
                psend(u->primary->handler,p);
                return;
            }
        }
    }


    /* unknown resources and all other crap handled by the unknown thread */
    psend(unknown_mp,p);
}

void psend(pth_msgport_t mp, jpacket p)
{
    jpq q;

    if(p == NULL || mp == NULL)
        return;

    q = pmalloc(p->p, sizeof(_jpq));
    q->p = p;

    pth_msgport_put(mp, (pth_message_t *)q);
}
