#include "jserver.h"

void roster_load(char *user, char *group, char *item, char *name)
{
    xmlnode next, newitem;

    if(user == NULL || group == NULL || item == NULL)
        return;
        
    debug("%s roster %s %s %s",user,group,item,name);

    next = xmlnode_get_firstchild(roster_get(user));
    while(next != NULL)
    {
        if(j_strcmp(xmlnode_get_attrib(next,"name"),group) == 0)
            break;
        next = xmlnode_get_nextsibling(next);
    }

    if(next == NULL)
    {
        next = xmlnode_insert_tag(roster_get(user),"group");
        xmlnode_put_attrib(next,"name",group);
    }

    newitem = xmlnode_insert_tag(next,"item");
    xmlnode_put_attrib(newitem,"jid",item);
    if(name != NULL)
        xmlnode_insert_cdata(newitem,name,strlen(name));
}

xmlnode roster_get(char *user)
{
    xmlnode roster;
    pool p;
    mapi m;
    udata u;
    
    if(user == NULL)
        return;

    u = user_data(user);
    if(u->roster == NULL)
    {
        u->roster = xmlnode_new_tag("roster");

        p = pool_new(); /* temporary pool for this module call */
        m = mapi_new(M_PHASE_ROSTER_INIT, p, NULL, NULL);
        m->user = pstrdup(p,user);
        module_dispatch(m); /* let the module initialize the user's roster */
        pool_free(p);
    }

    return u->roster;
}

/* internal use, rebuild subscriber list from the roster */
void subscriber_rebuild(char *user)
{
    udata u;
    xmlnode next, next2;
    pool p;

    u = user_data(user);
    if(u->subscribers != NULL)
        pool_free(u->subscribers->p);

    p = pool_new();

    next = xmlnode_get_firstchild(roster_get(user));
    while(next != NULL) /* groups */
    {
        next2 = xmlnode_get_firstchild(next);
        while(next2 != NULL) /* items within a group */
        {
            u->subscribers = jid_append(u->subscribers,jid_new(p, xmlnode_get_attrib(next2,"jid")));
            next2 = xmlnode_get_nextsibling(next2);
        }
        next = xmlnode_get_nextsibling(next);
    }
}

void session_roster_handle(session s, jpacket p)
{
    xmlnode query, next, next2, roster, presence;
    mapi m;
    
    roster = roster_get(s->r->user);
    query = xmlnode_get_tag(p->x, "query");    
    if(query == NULL)
        return;

    debug("SROSTER: %s",xmlnode2str(roster));
    
    if(!xmlnode_has_children(query))
    {
        m = mapi_new(M_PHASE_ROSTER_GET, p->p, s, p);
        m->group = NULL;
        module_dispatch(m);
        /* check to see if a module handled it or if we us internal roster stuff */
        if(m->result == M_DEFAULT)
        {
            next = xmlnode_get_firstchild(roster);
            while(next != NULL)
            {
                xmlnode_put_attrib(xmlnode_insert_tag(query,"group"),"name",xmlnode_get_attrib(next,"name"));
                next = xmlnode_get_nextsibling(next);
            }
        }
        return;
    }

    next = xmlnode_get_firstchild(query);
    m = mapi_new(M_PHASE_ROSTER_GET, p->p, s, p);
    while(next != NULL)
    {
        if(xmlnode_get_type(next) != NTYPE_TAG)
        {
            next = xmlnode_get_nextsibling(next);
            continue;
        }

        m->result = M_DEFAULT;
        m->group = NULL;
        m->item = NULL;
        m->name = NULL;
        if(j_strcmp(xmlnode_get_name(next),"add") == 0)
        {
            m->phase = M_PHASE_ROSTER_ADD;
            m->group = xmlnode_get_attrib(next,"group");
            m->item = xmlnode_get_attrib(next,"item");
            m->name = xmlnode_get_data(xmlnode_get_firstchild(next));
        }
        else if(j_strcmp(xmlnode_get_name(next),"del") == 0)
        {
            m->phase = M_PHASE_ROSTER_DEL;
            m->group = xmlnode_get_attrib(next,"group");
            m->item = xmlnode_get_attrib(next,"item");
        }
        else if(j_strcmp(xmlnode_get_name(next),"group") == 0)
        {
            m->phase = M_PHASE_ROSTER_GET;
            m->group = xmlnode_get_attrib(next,"name");
        }

        debug("roster/m %d:%s/%s/%s",m->phase,m->group,m->item,m->name);

        module_dispatch(m);

        if(m->result == M_DEFAULT)
        { /* no module handled it, use internal roster */
            if(m->phase == M_PHASE_ROSTER_ADD && m->group != NULL && m->item != NULL)
            {
                xmlnode_hide(next);
                roster_load(s->r->user, m->group, m->item, m->name);
                subscriber_rebuild(s->r->user);
                /* send built-in presence subscribe to the new entry */
                presence = xmlnode_new_tag("presence");
                xmlnode_put_attrib(presence,"to",m->item);
                xmlnode_put_attrib(presence,"type","subscribe");
                psend(s->mp_from,jpacket_new(presence));
            }else{
                next2 = xmlnode_get_firstchild(roster); /* descend into the list of groups */
                while(next2 != NULL)
                {
                    if(j_strcmp(xmlnode_get_attrib(next2,"name"),m->group) == 0)
                    {
                        if(m->phase == M_PHASE_ROSTER_GET)
                        {
                            /* copy the children into the result packet */
                            xmlnode_insert_node(next,xmlnode_get_firstchild(next2));
                        }else if(m->phase == M_PHASE_ROSTER_DEL){
                            /* descend into the group's items */
                            next2 = xmlnode_get_firstchild(next2);
                            while(next2 != NULL)
                            {
                                if(j_strcmp(xmlnode_get_attrib(next2,"jid"),m->item) == 0)
                                {
                                    xmlnode_hide(next2);
                                    xmlnode_hide(next);
                                    subscriber_rebuild(s->r->user);
                                    /* send built-in presence subscribe to the new entry */
                                    presence = xmlnode_new_tag("presence");
                                    xmlnode_put_attrib(presence,"to",m->item);
                                    xmlnode_put_attrib(presence,"type","unsubscribe");
                                    psend(s->mp_from,jpacket_new(presence));
                                    break;
                                }
                                next2 = xmlnode_get_nextsibling(next2);
                            }
                        }
                        break;
                    }
                    next2 = xmlnode_get_nextsibling(next2);
                }
            }
        }
        next = xmlnode_get_nextsibling(next);
    }
    debug("SROSTER: %s",xmlnode2str(roster));
}

